eval 'exec perl -w -S $0 ${1+"$@"}'
  if 0;

sub numerically { $a <=> $b; }

if ( $#ARGV < 0 ||
     grep( !/^[\w\.\/]+(=(-2|-1|\d+(\:\d+)?))?$/, @ARGV ) )
  {
    print <<'EOF';
#............................................................................
#...Name
#...====
#... procmon_plot
#...
#...Usage
#...=====
#... procmon_plot.pl <procmon output file list>
#...
#...Purpose
#...=======
#... Creates postscript plots from the procmon output file(s)
#... and saves them to "procmon.ps" and "procmon.pdf".
#...
#... This script uses gnuplot and ps2pdf (they must be in the PATH)
#...
#...Examples
#...========
#... 1) Plot data from one file (ie one process):
#...    procmon_plot.pl procmon_qsc20_11131568.txt
#... 2) Plot data from many files (ie all process from the run):
#...    procmon_plot.pl procmon_*
#...
#...Program Flow
#...============
#...1) Get names of files
#...2) From first file, get names of data fields
#...3) foreach data field, create plot
#............................................................................
EOF
  exit;
  }

my(
   $arch, # architecture you are on
   $datafile, # single data file
   @datafiles, # the data files
   $done, # if done or not
   $field_EpochTime, # the position of the EpochTime field
   @field_names, # split of PROCMON line
   %fields, # $fields{position} = name
   $gnuplot, # name of gnuplot command
   $gnuplot_commands, # the commands given to gnuplot
   $gnuplot_factor, # size factor for having 1st graph larger (X times larger)
   $gnuplot_origin, # origin for multiplot
   $gnuplot_size, # size for multiplot
   $i, # loop variable
   $ierr, # error value
   $line, # line of a file
   %name_key, # $name_key{short name} = long name
   $num_plots, # number of plots done so far
   $num_plots_total, # number of plots total
   $output, # output from shell command
   $output_file_ps, # gnuplot output file
   $output_file_pdf, # ps2pdf output file
   $plot_fields_skipped, # regexp of fields not to plot
   $plot_number, # which plot number you are on
   $plot_number_max, # maximum plot number per page
   $ps2pdf, # where ps2pdf is
  );
#..........................
#...initialize variables...
#..........................
$plot_fields_skipped = '\b(PROCMON|EpochTime|pWallTime)\b';
$gnuplot_commands = "";
$plot_number_max = 4;
$gnuplot_factor = 1 + .75*($plot_number_max/4); # scaled for font size 10
$output_file_ps = "procmon.ps";
$output_file_pdf = "procmon.pdf";
undef( %fields );
undef( @datafiles );
#................
#...find execs...
#................
$arch = `uname -a 2>&1`;
if( $arch =~ /alpha/i )
  {
    $ENV{PATH} = "/usr/local/gnu/bin:$ENV{PATH}";
    $ENV{PATH} = "/usr/projects/ups/ghostscript/alpha/bin:$ENV{PATH}";
  }
if( $arch =~ /irix/i )
  {
    $ENV{PATH} = "/usr/local/gnu/bin:$ENV{PATH}";
    $ENV{PATH} = "/usr/projects/ups/ghostscript/irix/bin:$ENV{PATH}";
  }
if( $arch =~ /linux/i )
  {
    $ENV{PATH} = "/usr/projects/ups/ghostscript/linux/bin:$ENV{PATH}";
  }
$gnuplot = &which_exec( "gnuplot" );
if( $gnuplot !~ /\S/ )
  {
    exit( -1 );
  }
$ps2pdf = &which_exec( "ps2pdf" );
#.......................
#...create @datafiles...
#.......................
foreach $datafile ( @ARGV )
  {
    push( @datafiles, $datafile );
    #.....................................
    #...check for existence of datafile...
    #.....................................
    if ( ! -T "$datafile" )
      {
        $ierr = -1;
        &print_error( 
                     "Non-text file or could not find datafile [$datafile].",
                     $ierr
                    );
        exit( $ierr );
      }
  }
if( $#datafiles < 0 )
  {
    $ierr = -1;
    &print_error( 
                 "Must supply list of datafiles",
                 $ierr
                );
    exit( $ierr );
  }
#.............................
#...DONE: create @datafiles...
#.............................
#...........................
#...create list of fields...
#...........................
$datafile = $datafiles[0];
if( ! open( FILE, "$datafile" ) )
  {
    $ierr = -1;
    &print_error( 
                 "Could not open datafile [$datafile]",
                 $ierr
                );
    exit( $ierr );
  }
$done = "false";
while( !eof(FILE) && $done ne "true" )
  {
    $line = <FILE>;
    if( $line =~ /^\#PROCMON:key\s+([^= ]+) = (.*)\n$/ )
      {
        $name_key{$1} = $2;
      }
    if( $line =~ /^\#PROCMON\s/ )
      {
        $done = "true";
        @field_names = split( /\s+/, $line );
        for( $i = 0; $i <= $#field_names; $i++ )
          {
            $fields{$i+1} = $field_names[$i];
          }
      }
  }
close( FILE );
$field_EpochTime = -1;
foreach $field ( keys %fields )
  {
    if( $fields{$field} eq "EpochTime" )
      {
        $field_EpochTime = $field;
        last;
      }
  }
if( $field_EpochTime < 0 )
  {
    $ierr = -1;
    &print_error(
                 "Could not find EpochTime field".
                 "(perhaps no '#PROCMON' line) in file [$datafile]",
                 "One can be created by running 'procmon' and",
                 "putting the #PROCMON line at the start of the file.",
                 $ierr
                );
    exit( $ierr );
  }
#.................................
#...DONE: create list of fields...
#.................................
#....................................
#...create header gnuplot_commands...
#....................................
$gnuplot_commands = "";
$gnuplot_commands .= "set output '$output_file_ps'; ";
$gnuplot_commands .= "set terminal postscript portrait color 'Times-Roman' 10; ";
#..........................................
#...DONE: create header gnuplot_commands...
#..........................................
#................................
#...create plots for each type...
#................................
$num_plots_total = 0;
foreach $field( sort numerically keys %fields )
  {
    if( $fields{$field} !~ /$plot_fields_skipped/ )
      {
        $num_plots_total++;
      }
  }
$plot_number = 0;
$num_plots = 0;
foreach $field ( sort numerically keys %fields )
  {
    if( $fields{$field} =~ /$plot_fields_skipped/ )
      {
        next;
      }
    printf( "  Plot %3d:", $num_plots+1 );
    printf( " %9s = %s\n", $fields{$field}, $name_key{$fields{$field}} );
    #........................................
    #...reset values used for rest of page...
    #........................................
    if( $plot_number == 0 )
      {
        $gnuplot_commands .= "set multiplot;";
        $gnuplot_commands .= "set nokey; ";
        $gnuplot_commands .= "set grid x2tics ytics; ";
        $gnuplot_commands .= "set noxtics; ";
        $gnuplot_commands .= "set x2tics; ";
        $gnuplot_commands .= "set x2label 'Time (secs)'; ";
        $gnuplot_commands .= "set x2tics rotate; ";
        $gnuplot_commands .= 'set format x2 "%17f"; ';
        $gnuplot_commands .= "set ytics; ";
        $gnuplot_commands .= "set ylabel 'Value'; ";
        $gnuplot_commands .= 'set format y "%12.5e"; ';
        $gnuplot_size = $gnuplot_factor/($gnuplot_factor+$plot_number_max-1);
        $gnuplot_origin = 1 - $gnuplot_size;
      }
    else
      {
        $gnuplot_commands .= "set x2label ''; ";
        $gnuplot_commands .= 'set format x2 ""; ';
        $gnuplot_size = 1/($gnuplot_factor+$plot_number_max-1);
        $gnuplot_origin -= $gnuplot_size;
      }
    $gnuplot_commands .= "set size 1,$gnuplot_size ; ";
    $gnuplot_commands .= "set origin 0, $gnuplot_origin; ";
    $gnuplot_commands .=
      "set title '$fields{$field} = $name_key{$fields{$field}}'; ";
    $gnuplot_commands .= "plot ";
    foreach $datafile ( @datafiles )
      {
        $gnuplot_commands .= "'$datafile' ";
        $gnuplot_commands .= "using ".$field_EpochTime.":".$field." ";
        $gnuplot_commands .= "notitle ";
        $gnuplot_commands .= "with lines ";
        $gnuplot_commands .= ", ";
      }
    $gnuplot_commands =~ s/, $/;/;
    $plot_number++;
    $num_plots++;
    if( $plot_number >= $plot_number_max || $num_plots == $num_plots_total )
      {
        $gnuplot_origin = 1;
        $plot_number = 0;
        $gnuplot_commands .= "set nomultiplot; ";
      }
  }
print "  Number of plots: $num_plots\n";
#.................
#...run gnuplot...
#.................
if( $gnuplot ne "" )
  {
    printf " Creating [$output_file_ps]\n";
    open ( FILE, "|$gnuplot > /dev/null 2>&1" );
    print FILE $gnuplot_commands;
    close FILE;
    if( $ps2pdf ne "" && -T "$output_file_ps" )
      {
        printf " Creating [$output_file_pdf]\n";
        $output = `$ps2pdf $output_file_ps $output_file_pdf 2>&1`;
        print $output;
      }
  }

#.........
#...end...
#.........
exit( 0 );

#............................................................................
#...Name
#...====
#... print_error
#...
#...Purpose
#...=======
#... Print a standard error message.
#...
#...Arguments
#...=========
#... error_lines  Intent: in
#...              Perl type: array
#...              0: cause of error (file_name:line_number)
#...              1: explanation/fix (if there is one)
#...              2: error value
#...
#...Program Flow
#...============
#... 1) see if warning or error (last argument is 0 or not)
#... 2) Line up error lines by column
#... 3) find out who was calling this routine and the line number
#... 4) print out info
#............................................................................
sub print_error
  {
    my( 
       @error_lines # what emitted the error
      ) = @_;
    my(
       $error_level,  # what is printed (eg WARNING or ERROR)
       $error_line,   # each line of input argument
       @routine_info, # var from caller
       $routine_name, # routine name
       $spaces        # spaces for lining up columns
      );
    #......................................................
    #...assign WARNING or ERROR depending on error value...
    #......................................................
    if ( "$error_lines[$#error_lines]" eq "0" )
      {
        $error_level = "**WARNING**";
        $spaces      = "           ";
      }
    else
      {
        $error_level = "**ERROR**";
        $spaces      = "         ";
      }
    #............................................................
    #...DONE: assign WARNING or ERROR depending on error value...
    #............................................................
    #.......................................
    #...init error and add argument lines...
    #.......................................
    $error_line = shift( @error_lines );
    $error_message = "\n$error_level $error_line\n";
    foreach $error_line ( @error_lines )
      {
        $error_message .= "$spaces $error_line\n";
      }
    #............................................................
    #...get routine name (special if called from main program)...
    #............................................................
    @routine_info = caller(1);
    if ( $#routine_info < 3 )
      {
        $routine_name = $0;
      }
    else
      {
        $routine_name = (caller(1))[3];
      }
    $routine_name = $routine_name.":".(caller(0))[2];
    #...........................
    #...postpend routine name...
    #...........................
    $error_message .= "$spaces $routine_name\n";
    #.................
    #...print error...
    #.................
    print STDERR $error_message;
  }
#............................................................................
#...Name
#...====
#... which_exec
#...
#...Purpose
#...=======
#... Returns value for full path to executable ("" if not found).
#...
#...Arguments
#...=========
#... exec Intent: in
#...      Perl type: string scalar
#...
#...Program Flow
#...============
#... 1) go through path to find exec
#............................................................................
sub which_exec
  {
    my( 
       $exec, # exec to find
      ) = @_;
    my(
       $ierr, # error return value
       $exec_try, # see if exec is here
       $found, # if found
       $path, # current dir in search for execs
       @paths, # list of search paths for execs
       $this_dir, # current directory
      );
    #.................................................
    #...build paths from PATH and current directory...
    #.................................................
    $path = $ENV{PATH};
    @paths = split( /:/, $ENV{PATH} );
    ($this_dir = $0) =~ s/\/[^\/]+$//;
    unshift( @paths, $this_dir );
    #.....................................
    #...loop through paths to find exec...
    #.....................................
    $found = "false";
    foreach $path (@paths)
      {
        $exec_try = "$path/$exec";
        if( -x "$exec_try" )
          {
            $found = "true";
            last;
          }
      }
    #........................................
    #...error if still could not find exec...
    #........................................
    if( $found eq "false" )
      {
        $ierr = -1;
        &print_error(
                     "Necessary executable [$exec] not found in PATH",
                     $ierr
                    );
        $exec_try = "";
      }
    return( $exec_try );
  }
